#include <linux/module.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/irq.h>
#include <linux/interrupt.h>
#include <linux/mm.h>
#include <linux/fs.h>
#include <linux/of_platform.h>
#include <linux/of.h>

#include <asm/io.h>
#include <asm/irq.h>

static int irq_num = 0;

static irqreturn_t pwr_fail_irqhandler(int irq, void *dev_id)
{
	/* Remount all FS read only */
	emergency_remount();

	/* Do nothing */
	return IRQ_HANDLED;
}

static int pwr_fail_probe(struct of_device *ofdev, const struct of_device_id *match)
{
	struct device_node *np = ofdev->node;

	if (irq_num != 0) {
		printk(KERN_ERR "pwr_fail: Already in use\n");
		return -ENODEV;
	}
	
	irq_num = irq_of_parse_and_map(np, 0);
	
	/* Request IRQ */
	if (request_irq(irq_num, pwr_fail_irqhandler, 0, "pwrfail_irq", NULL) != 0) {
		printk(KERN_ERR "pwr_fail: Error registering IRQ\n");
		return -ENODEV;
	}
	
	printk("EchoCPUV2 power fail interrupt driver registered\n");

	return 0;
}

static int __devexit pwr_fail_remove(struct of_device *dev)
{
	free_irq(irq_num, NULL);
	irq_num = 0;
	return 0;
}

static struct of_device_id pwr_fail_match[] =
{
	{
		.compatible = "pwrfail",
	},
	{},
};

/* Structure for a device driver */
static struct of_platform_driver pwr_fail_driver = {
	.name = "echocpu-pwrfail",
	.match_table = pwr_fail_match,

	.probe = pwr_fail_probe,
	.remove = pwr_fail_remove,
};

static int __init pwr_fail_init(void)
{
	int err;

	err = of_register_platform_driver(&pwr_fail_driver);

	return err;
}

static void __exit pwr_fail_exit(void)
{
	of_unregister_platform_driver(&pwr_fail_driver);
}

module_init(pwr_fail_init);
module_exit(pwr_fail_exit); 
