#include <linux/module.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/irq.h>
#include <linux/interrupt.h>
#include <linux/mm.h>
#include <linux/of_platform.h>
#include <linux/of.h>

#include <asm/io.h>
#include <asm/irq.h>
#include <asm/uaccess.h>

struct test_isa_drv_data {
	int irq;
};

struct test_isa_drv_data drv_data;

static irqreturn_t test_isa_irqhandler(int irq, void *dev_id)
{
	/* Do nothing */
	printk("ISA board\n");
	return IRQ_HANDLED;
}

static int test_isa_probe(struct of_device *ofdev, const struct of_device_id *match)
{
	struct resource io_resource;
	struct resource mem_resource;
	struct device_node *np = ofdev->node;
	int ret;
#if 0
	void *io_res;
#endif
	
	printk("Probin ISA device\n");
	
	ret = of_address_to_resource(np, 0, &io_resource);
	if (ret) {
		printk(KERN_ERR "invalid address from OF\n");
		return ret;
	}
	printk("IO range is 0x%08x to 0x%08x\n", (u32)io_resource.start, (u32)io_resource.end);
	
	ret = of_address_to_resource(np, 1, &mem_resource);
	if (ret) {
		printk(KERN_ERR "invalid address from OF\n");
		return ret;
	}
	printk("MEM range is 0x%08x to 0x%08x\n", (u32)mem_resource.start, (u32)mem_resource.end);
	
	drv_data.irq = irq_of_parse_and_map(np, 9);
	printk("IRQ is at %d\n", drv_data.irq);
	
	/* Request IRQ */
	if (request_irq(drv_data.irq, test_isa_irqhandler, IRQF_TRIGGER_RISING/*IRQ_TYPE_LEVEL_HIGH*/, "test_isa_irq", NULL) != 0) {
		printk("Error registering IRQ\n");
		return -ENODEV;
	}

#if 0
	io_res = ioremap(io_resource.start, (io_resource.end - io_resource.start) + 1);
	if (io_res == NULL) {
		printk(KERN_ERR "Error ioremap\n");
	} else {
		unsigned int i;

		printk("IO space is at 0x%p\n", io_res);
		for (i=0; i<0x300; i++) {
			if ((i & 0xF) == 0 && i!= 0) {
				printk("\n");
			}
			printk("0x%02x ", in_8( io_res + i ));
		}
		printk("\n");
		iounmap(io_res);
	}
#endif
	
	return 0;
}

static int __devexit test_isa_remove(struct of_device *dev)
{
	free_irq(drv_data.irq, NULL);
	return 0;
}

static struct of_device_id test_isa_match[] =
{
	{
		.type = "isa",
		.compatible = "isadev",
	},
	{},
};

/* Structure for a device driver */
static struct of_platform_driver test_isa_driver = {
	.name = "fsl-isadev",
	.match_table = test_isa_match,

	.probe = test_isa_probe,
	.remove = test_isa_remove,
};

static int __init test_isa_init(void)
{
	int err;

	printk("ISA Test driver version 0.1\n");
	
	err = of_register_platform_driver(&test_isa_driver);

	return err;
}

static void __exit test_isa_exit(void)
{
	of_unregister_platform_driver(&test_isa_driver);
}

module_init(test_isa_init);
module_exit(test_isa_exit);
